package cn.timer.api.controller.disk.sevice;

import cn.timer.api.bean.disk.DiskCatalogue;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @author wuqingjun
 * @email 284718418@qq.com
 * @date 2021/12/29
 */
@Service
public class DiskCatalogueServiceImpl implements DiskCatalogueService {

    private static List<DiskCatalogue> treeList = new ArrayList<>();

    @Override
    public List<DiskCatalogue> getDiskCatalogueList(Integer orgId, Integer userId, Integer type) {
        treeList = DiskCatalogue.builder().build().selectList(new QueryWrapper<DiskCatalogue>()
                .lambda().eq(DiskCatalogue::getOrgId, orgId)
                .eq(DiskCatalogue::getCreateUserId, userId)
                .eq(DiskCatalogue::getType, type)
        );
        List<DiskCatalogue> list = treeList.stream().filter(s -> s.getParentId() == 0).collect(Collectors.toList());

        return getTree(list);
    }

    /**
     * 递归获取树形结构
     *
     * @param parentList 一级节点
     * @return
     */
    public static List<DiskCatalogue> getTree(List<DiskCatalogue> parentList) {
        List<DiskCatalogue> list = new ArrayList<>();
        if (parentList != null) {
            for (int i = 0; i < parentList.size(); i++) {
                list.add(recursiveTree(parentList.get(i).getId()));
            }
        }
        return list;
    }


    /**
     * 递归算法解析成树形结构
     *
     * @param cid
     */
    public static DiskCatalogue recursiveTree(Integer cid) {
        DiskCatalogue node = getMenuById(cid);
        List<DiskCatalogue> childTreeNodes = getChildTreeById(cid);
        for (DiskCatalogue child : childTreeNodes) {
            DiskCatalogue n = recursiveTree(child.getId());
            List<DiskCatalogue> list = new ArrayList<DiskCatalogue>();
            list.add(n);
            if (node.getChildren() == null) {
                List<DiskCatalogue> li = new ArrayList<DiskCatalogue>();
                node.setChildren(li);
            }
            node.getChildren().add(n);
        }
        return node;
    }

    /**
     * 根据CID查询节点对象
     */
    public static DiskCatalogue getMenuById(Integer cid) {
        Map map = getTreeMap();
        return (DiskCatalogue) map.get(cid);
    }

    /**
     * 一次性取所有数据，为了减少对数据库查询操作
     *
     * @return
     */
    public static Map getTreeMap() {
        Map map = new HashMap<Integer, DiskCatalogue>();
        if (null != treeList) {
            for (DiskCatalogue d : treeList) {
                map.put(d.getId(), d);
            }
        }
        return map;
    }

    /**
     * 根据父节点CID获取所有了节点
     */
    public static List<DiskCatalogue> getChildTreeById(Integer cid) {
        List<DiskCatalogue> list = new ArrayList<>();
        if (null != treeList) {
            for (DiskCatalogue d : treeList) {
                if (null != cid) {
                    if (cid.equals(Integer.valueOf(d.getParentId()))) {
                        list.add(d);
                    }
                }
            }
        }
        return list;
    }

}
